import serial

from .exceptions import RunError
from .logging import get_logger

logger = get_logger()


def get_operator(modem_console_path):
    """ Example output
    AT+COPS?
    +COPS: 0,0,"T-Mobile CZ T-Mobile CZ",7
    """
    try:
        with serial.Serial(modem_console_path) as cons:
            cons.write(b"AT+COPS?\r")
            cons.readline()
            line = cons.readline().decode("utf-8")
            parts = line.split(",")
            if len(parts) >= 3:
                return "".join(parts[2].split()[0].split('"'))
            else:
                raise RunError("Wrong console output ({})".format(line))
    except serial.serialutil.SerialException as e:
        raise RunError(str(e)) from e


def get_gstatus(console):
    console.write(b"AT!GSTATUS?\r")
    value_map = {
        "LTE bw": "Bandwidth",
        "PCC RxM RSSI": "RSSI",
        "PCC RxM RSRP": "RSRP",
        "RSRQ (dB)": "RSRQ",
        "SINR (dB)": "SINR Rx[0]",
    }
    gstatus = {}
    while True:
        line = console.readline().decode("utf-8")
        if len(line) == 4:  # this is "OK\r\n"
            break
        for value_pair in line.split("\t"):
            value_pair = value_pair.split(":")
            value_name = value_map.get(value_pair[0], None)
            if value_name:
                gstatus[value_name] = value_pair[1].split()[0]

    return gstatus


def get_lteinfo(console):
    lteinfo = {}
    console.write(b"AT!LTEINFO?\r")
    while True:
        line = console.readline().decode("utf-8")
        if len(line) < 3:
            continue
        split_line = line.split()
        if len(line) == 4:  # this is "OK\r\n"
            break
        if split_line[0] == "Serving:":
            line = console.readline()
            split_line = line.split()
            lteinfo["MCC"] = int(split_line[1])
            lteinfo["MNC"] = int(split_line[2])
            lteinfo["LAC"] = int(split_line[3])
            lteinfo["Cell Id"] = int(split_line[4], 16)
            lteinfo["PCI"] = int(split_line[9])
        if split_line[0] == "InterFreq:" and split_line[1] == "EARFCN":
            line = console.readline()
            try:
                lteinfo["DL EARFCN"] = int(line.split()[0])
            except IndexError:
                lteinfo["DL EARFCN"] = 0

    return lteinfo


def get_gpsloc(console):
    console.write(b"AT!GPSLOC?\r")
    console.readline()
    while True:
        line = console.readline().decode("utf-8")
        if len(line) == 4:  # this is "OK\r\n"
            return {}
        line_split = line.split(":")
        if line_split[0] == "Lat":
            line_split = line_split[1].split()
            lat = (float(line_split[0]) +
                   float(line_split[2]) / 60 +
                   float(line_split[4]) / 3600)
            if line_split[6] == "S":
                lat = -lat
            continue
        if line_split[0] == "Lon":
            line_split = line_split[1].split()
            lon = (float(line_split[0]) +
                   float(line_split[2]) / 60 +
                   float(line_split[4]) / 3600)
            if line_split[6] == "W":
                lon = -lon
            break

    return {"lat": lat, "lon": lon}


def parse_lte_console(console):
    lteinfo = dict()

    lteinfo.update(get_gstatus(console))
    lteinfo.update(get_lteinfo(console))
    lteinfo.update(get_gpsloc(console))

    return lteinfo


def measure_lte(modem_console_path):
    try:
        with serial.Serial(
            modem_console_path,
            baudrate=115200,
            timeout=5
        ) as ser:
            return parse_lte_console(ser)
    except Exception as e:
        logger.error("Lteinfo acquire failed (%s)", e)
        return {}
